package com.god.business.common.domain;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.god.business.common.domain.request.ReqAddBreakDown;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * 故障记录
 *
 * @author LouZhiSHuo
 * @Date 2023/11/29 14:40
 **/
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class BreakDown {


    /**
     * 故障记录主建
     */
    private Integer id;

    /**
     * 车辆VIN
     */
    @NotEmpty(message = "车辆VIN不能为空")
    private String carVin;

    /**
     * 故障码Id
     */
    @NotNull(message = "故障主建不能为空")
    private Integer breakdownId;


    /**
     * 故障状态
     */
    private Integer status;


    /**
     * 故障开始时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH-mm-ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH-mm-ss",timezone = "GMT+8")
    private Date createTime;

    /**
     * 故障结束时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH-mm-ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH-mm-ss",timezone = "GMT+8")
    private Date endTime;


    /**
     * 创建人
     */
    private String createBy;

    /**
     * 修改人
     */
    private String updateBy;

    /**
     * 更新时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH-mm-ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH-mm-ss",timezone = "GMT+8")
    private Date updateTime;



    public static BreakDown insertBreakDown(ReqAddBreakDown reqAddBreakDown){
            return BreakDown.builder()
                    .carVin(reqAddBreakDown.getCarVin())
                    .breakdownId(reqAddBreakDown.getBreakdownId())
                    .status(reqAddBreakDown.getStatus())
                    .createTime(reqAddBreakDown.getCreateTime())
                    .endTime(reqAddBreakDown.getEndTime())
                    .createBy(reqAddBreakDown.getCreateBy())
                    .updateBy(reqAddBreakDown.getUpdateBy())
                    .updateTime(reqAddBreakDown.getUpdateTime())
                    .build();
    }

}
