package com.god.base.server.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.god.base.domain.Car;
import com.god.base.domain.Fence;
import com.god.base.remote.RemoteCarService;
import com.god.base.server.mapper.AlarmMapper;
import com.god.base.server.service.AlarmService;
import com.god.base.server.util.FenceAlgorithm;
import com.god.business.common.domain.BreakDown;

import com.god.common.core.domain.Result;

import lombok.extern.log4j.Log4j2;

import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.awt.geom.Point2D;
import java.util.*;

/**
 * @author LouZhiSHuo
 * @Date 2023/11/27 20:40
 **/
@Log4j2
@Service
public class AlarmServiceImpl extends ServiceImpl<AlarmMapper, BreakDown> implements AlarmService  {

    @Resource
    private RemoteCarService remoteCarService;

    @Resource
    private FenceAlgorithm fenceAlgorithm;






    /**
     * 围栏报警业务实现
     * @param carVinId
     */
    @Override
    public void startAlarm(String carVinId) {
        //调用 根据车辆VIN查询车辆信息 carController/list
        Result<Car> carResult = remoteCarService.list(carVinId);
        //车辆信息
        Car car = carResult.getData();
        //根据车辆信息查询该车辆绑定的围栏Id
        long fenceId = car.getFenceId();
        //获取到围栏Id,根据围栏Id查询围栏 信息 FenceController/fenceById
        //Redis中的围栏坐标信息格式: " 经度,维度; "
        Result<Fence> fenceResult = remoteCarService.fenceById((int) fenceId);
        //围栏信息
        Fence fence = fenceResult.getData();
        //获取该围栏行驶状态
        Integer driveStatus = fence.getDriveStatus();// 1-驶入 2-驶出
        //获取围栏坐标串
        String fenceLocation = fence.getFenceLocation(); //116.357183,39.926512; 116.394348,39.909989; 116.34757,39.90617;
        String[] split = fenceLocation.split(";");

        List<Point2D.Double> polygon = new ArrayList<>();

        for (int i = 0; i < split.length; i++) {
            String[] splits = split[i].split(",");

            Point2D.Double aDouble = new Point2D.Double();

            for (int j = 0; j < splits.length; j++) {
                if (j == 0){
                    aDouble.x = Double.parseDouble(splits[j]);
                } else {
                    aDouble.y = Double.parseDouble(splits[j]);
                }
            }
            polygon.add(aDouble);
        }

        //消费消息的消息

        //算法计算
        //假设当前经纬度
        Point2D.Double car_one = new Point2D.Double(39.9125, 116.3917);
        boolean ptInPoly = fenceAlgorithm.computeFence(car_one, polygon);

        if (driveStatus == 1){
            //驶入
                //围栏区域外 -正常 进入后报警
            if (ptInPoly) {
                //报警
                //存储消息到redis

            } else {
                log.info("当前车辆:{},正常行驶中~~~~~~",carVinId);
            }

        } else {
            //驶出
                //围栏区域内 -正常 驶出报警

            if (!ptInPoly) {
                //报警
            } else {
                log.info("当前车辆:{},正常行驶中~~~~~~",carVinId);
            }

        }
        //判断围栏状态
        //1-驶入
        //判断坐标是否在范围 外围:不做操作 , 坐标进入围栏内: 发送消息给围栏内的用户(使用RabbitMq)
        //2-驶出
        //判断坐标是否在范围 外围:发送消息给围栏内的用户(使用RabbitMq) , 坐标在围栏内:不做操作
    }


    /**
     * 添加故障记录
     * @param entity
     * @return
     */
    @Override
    public boolean save(BreakDown entity) {
        entity.setCreateTime(new Date());

        boolean save = super.save(entity);
        if (!save) {
            throw new RuntimeException("故障记录添加失败");
        }
        return true;
    }
}
