package com.god.base.server.util;

import org.springframework.stereotype.Component;

import java.awt.geom.Point2D;
import java.util.List;

/**
 * 围栏算法
 *
 * @author LouZhiSHuo
 * @Date 2023/11/28 19:12
 **/
@Component
public class FenceAlgorithm {


    /**
     * 围栏区域内计算
     * @param point
     * @param pts
     * @return
     */
    public boolean computeFence(Point2D.Double point, List<Point2D.Double> pts){
        //围栏顶点坐标数
        int fenceSize = pts.size();

        //如果点位在多边形的顶点或边上,也算在多边形内部,返回true
        boolean boundDrVertex  = true;

        //射线与多边形相交次数
        int intersectCount = 0;

        ///浮点型计时候的容差
        double precision = 2e-10;

        //相邻两点位置
        Point2D.Double fenceOne,fenceTwo;

        //确认当前点
        Point2D.Double p = point;

        //起点
        fenceOne = pts.get(0);

        for (int i =0;i<fenceSize;i++) {
            //判断当前点是否在围栏范围边界
            if (p.equals(fenceOne)) {
                //返回 true
                return boundDrVertex;
            }

            //终点
            fenceTwo = pts.get(i % fenceSize);

            //判断是否无交汇集
            if (p.x < Math.min(fenceOne.x,fenceTwo.x) || p.x > Math.max(fenceOne.x,fenceTwo.x)) {
                //无交点
                fenceOne = fenceTwo;
                //继续判断
                continue;
            }

            if (p.x > Math.min(fenceOne.x,fenceTwo.x) && p.x < Math.max(fenceOne.x,fenceTwo.x)) {
                //当前点在线段于X轴上的投影内
                if (p.y <= Math.max(fenceOne.y,fenceTwo.y)) {
                    //当前点的y坐标小于线段对y轴投影的最大值
                    if (fenceOne.x == fenceTwo.x && p.y >= Math.min(fenceOne.y,fenceTwo.y)) {
                        //当前点垂直在x轴边上
                        return boundDrVertex;
                    }

                    if (fenceOne.y == fenceTwo.y) {
                        //平行于X轴
                        if (fenceOne.y == p.y) {
                            //位于当前围栏的边上
                            return boundDrVertex;
                        } else {
                            //反之相交次数+1
                            ++intersectCount;
                        }

                    } else {
                        //非水平情况
                        double v = (p.x - fenceOne.x) * (fenceTwo.y - fenceOne.y) / (fenceTwo.x - fenceOne.x) + fenceOne.y;
                        //围栏某条边上
                        if (Math.abs(p.y - v) < precision) {
                            return boundDrVertex;
                        }

                        if (p.y < v) {
                            //相交次数+1
                            ++intersectCount;
                        }
                    }
                }
            } else {
                //当前带你不在线段投影到x轴的区间中
                if (p.x == fenceTwo.x && p.y <= fenceTwo.y) {
                    //位于x坐标对应的平行于y轴上的线上的低于终点y的坐标
                    Point2D.Double temp = pts.get((i + 1) % fenceSize);
                    if (p.x >= Math.min(fenceOne.x , temp.x) && p.x <= Math.max(fenceOne.x,temp.x)) {
                        //若当前点的x坐标位于fenceOne 和 temp 组成的线段关于x轴的投影中
                        //则记为该点的射线只穿过端点一次
                        //那么相交+1
                        ++intersectCount;
                    } else  {
                        //若当前坐标的x不包含 fenceOne 和 temp 组成的线段关于x轴的投影中 , 则点射线通过的两条线段组成了一个弯折的部分,
                        //那么记为该点的射线穿过了端点两次
                        intersectCount += 2;
                    }
                }
            }
            fenceOne = fenceTwo;
        }
        if (intersectCount % 2 == 0) {
            //偶数在围栏外面
            return false;
        } else  {
            //奇数在围栏内
            return true;
        }
    }
}
